function [neum_eigs, dir_eigs, Ss, ND1, DI]=sp_expansion_oct05(cx,cy,ax,ay,ell,lam_max,lam_0,Nmax,nrefine)
%%%
%%%
%%%

%%%
%%% geometry
%%%

% Use of geometry matrix:
% %%%%%%%%%%%%%%%%%%%%%%%
%
% Each column describes a curve in R^2.
%
% A column starting with a 2 describes a straight line; a column starting
% with a 4 describes an elliptical arc. In each case, the curve in question
% goes from (g(2,*),g(4,*)) to (g(3,*),g(5,*)).
%
% The equation of an elliptical arc is
%
% ((x-g(8,*))/g(10,*))^2 + ((y-g(9,*))/g(11,*))^2 = 1.
% 
% The entries in rows 6 and 7 describe the orientation.
%
g = [...
    2.0000    2.0000    2.0000    2.0000     2.0000    4.0000     4.0000;...
         0       ell       ell         0          0    0.0000      cx+ax;...
       ell       ell         0         0          0     cx+ax     0.0000;...
   -1.0000   -1.0000    1.0000    1.0000      cy-ay     cy-ay          0;...
   -1.0000    1.0000    1.0000     cy+ay    -1.0000         0      cy+ay;...
    1.0000    1.0000    1.0000    1.0000     1.0000         0          0;...
         0         0         0         0          0    1.0000     1.0000;...
         0         0         0         0          0        cx         cx;...
         0         0         0         0          0        cy         cy;...
         0         0         0         0          0        ax         ax;...
         0         0         0         0          0        ay         ay;...
         0         0         0         0          0         0          0];

npieces=size(g,2);

%%%
%%% computing Dirichlet eigenvalues
%%%

b=bdir_0(7);
%
[p,e,t]=initmesh(g);
for k = 1:nrefine-1
[p,e,t]=refinemesh(g,p,e,t);
end;
[p1,e1,t1]=refinemesh(g,p,e,t);
figure(1), clf, pdemesh(p,e,t), axis equal, title('Uniform mesh');

[v,dir_eigs]=pdeeig(b,p1,e1,t1,1.0,0.,1.0,[-1 lam_max]);
dir_eigs;



%%%
%%% computing Neumann eigenvalues
%%%

b=bneum_0(7);

%%%
%%% fixing M for scaling
%%%
[K1,M1,F1]=assema(p1,t1,0.,1.,0.);
%
[v,neum_eigs]=pdeeig(b,p1,e1,t1,1.0,0.,1.0,[-1 lam_max]);
neum_eigs;
N_eigs=length(neum_eigs);
for m=1:N_eigs
   v(:,m)=(v(:,m)'*M1*v(:,m))^(-1/2)*v(:,m);
end;

%
%
%
p_indices=find(p1(1,:)==ell);
[y_values, ix]=sort(p1(2, p_indices));
%

%%
%%% assembling basis functions Psi
%%%

Psi=zeros(length(p_indices), Nmax);
Psi(:,1)=sqrt(0.5);
for n=2:Nmax
   Psi(:,n)=cos((n-1)*pi/2*(y_values+1)');
end;
%%%

%%%
%%% integrating for each eigenfunction
%%%

Ss=zeros(N_eigs,Nmax);

for m=1:N_eigs
   v_b=v(p_indices,m)';
   for n=1:Nmax
      Ss(m,n)=trapz(y_values', v_b(ix)'.*Psi(:,n));
	end;
end;

for n = 1:Nmax
% Solve PDE with nth basis function as Neumann data:
    b = bmixed(2,[1, 3, 4, 5, 6, 7],n);
    u(:,n) = assempde(b,p1,e1,t1,1.0,-lam_0,0.0);
%    if n == 2 
%        pdeplot(p1,e1,t1,'xydata',u(:,n),'zdata',u(:,n),'mesh','on');
%        keyboard;
%    end;
end;
% Recover solution on boundary and integrate:
for n = 1:Nmax
    ubound = u(p_indices,n)';
%    if n==2
%        plot(y_values,ubound(ix),'*');
%        keyboard;
%    end;
    for m = 1:Nmax
%        ND1(n,m) = trapz(y_values',ubound(ix).*cos((m-1)*(pi/2)*(1+y_values)));
        ND1(n,m) = trapz(y_values',ubound(ix)'.*Psi(:,m));
%        if (m==1 && n==1) 
%        plot(y_values,Psi(:,m),'o'); 
%       keyboard;
%        hold on;
%        plot(y_values,cos((m-1)*(pi/2)*(1+y_values)),'*');
%        keyboard;
%        hold off;
%        end;
    end;
end;
%ND1(:,1) = ND1(:,1)/2;

DI = Ss'*inv(diag(neum_eigs-lam_0))*Ss;

